@@ -53,6 +53,40 @@ class GtkObject:
 			return cmp(self._o, other._o)
 		else:
 			return cmp(id(self), id(other))
+	def __getattr__(self, attr):
+		# this function allows setting attributes on an object so that
+		# they will always be available with the object.  Due to
+		# reference counting problems, we can't always pass the
+		# same GtkObject instance to a callback.
+		#if attr[0] == '_' or not self.__dict__.has_key('_o'):
+		#	raise AttributeError, attr
+		#dict = self.get_data('Python-Attributes')
+		#if dict and dict.has_key(attr):
+		#	return dict[attr]
+		raise AttributeError, attr
+	#def __setattr__(self, attr, value):
+	#	if attr[0] == '_' or self.__dict__.has_key(attr) or \
+	#	   not self.__dict__.has_key('_o'):
+	#		self.__dict__[attr] = value
+	#	dict = self.get_data('Python-Attributes')
+	#	if not dict:
+	#		dict = {}
+	#		self.set_data('Python-Attributes', dict)
+	#	dict[attr] = value
+	#def __delattr__(self, attr):
+	#	if self.__dict__.has_key(attr):
+	#		del self.__dict__[attr]
+	#		return
+	#	if not self.__dict__.has_key('_o'):
+	#		raise AttributeError, \
+	#		      'delete non-existing instance attribute'
+	#	dict = self.get_data('Python-Attributes')
+	#	if dict and dict.has_key(attr):
+	#		del dict[attr]
+	#	else:
+	#		raise AttributeError, \
+	#		      'delete non-existing instance attribute'
+			
 	def flags(self, mask=None):
 		if mask:
 			return _gtk.GTK_OBJECT_FLAGS(self._o) & mask
@@ -133,7 +167,7 @@ class GtkAdjustment(GtkData):
 		}
 		if attr in attrs.keys():
 			return attrs[attr](self._o)
-		raise AttributeError, attr
+		return GtkData.__getattr__(self, attr)
 	def set_value(self, value):
 		_gtk.gtk_adjustment_set_value(self._o, value)
 	def clamp_page(self, lower, upper):
@@ -154,7 +188,7 @@ class GtkTooltips(GtkData):
 		_gtk.gtk_tooltips_disable(self._o)
 	def set_delay(self, delay):
 		_gtk.gtk_tooltips_set_delay(self._o, delay)
-	def set_tip(self, w, tip, tip_private):
+	def set_tip(self, w, tip, tip_private=None):
 		_gtk.gtk_tooltips_set_tip(self._o, w._o, tip, tip_private)
 	def set_tips(self, w, tip):
 		print "GtkTooltip.set_tips deprecated -- use set_tip instead"
@@ -272,6 +306,8 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_lock_accelerators(self._o)
 	def get_ancestor(self, type):
 		return _obj2inst(_gtk.gtk_widget_get_ancestor(self._o, type))
+	def get_child_requisition(self):
+		return _gtk.gtk_widget_get_child_requisition(self._o)
 	def get_colormap(self):
 		return _gtk.gtk_widget_get_colormap(self._o)
 	def get_composite_name(self):
@@ -454,7 +490,8 @@ class GtkContainer(GtkWidget):
 	def resize_children(self):
 		_gtk.gtk_container_resize_children(self._o)
 	def set_focus_child(self, child):
-		_gtk.gtk_container_set_focus_child(self._o, child._o)
+		if child: child = child._o
+		_gtk.gtk_container_set_focus_child(self._o, child)
 	def set_focus_vadjustment(self, adj):
 		_gtk.gtk_container_set_focus_vadjustment(self._o, adj._o)
 	def set_focus_hadjustment(self, adj):
@@ -529,7 +566,7 @@ class GtkToggleButton(GtkButton):
 	    }
 	    if attrs.has_key(attr):
 		    return attrs[attr](self._o)
-	    raise AttributeError, attr
+	    return GtkButton.__getattr__(self, attr)
 	def set_mode(self, di):
 		_gtk.gtk_toggle_button_set_mode(self._o, di)
 	def set_active(self, active):
@@ -599,6 +636,10 @@ class GtkHandleBox(GtkBin):
 		self._o = _gtk.gtk_handle_box_new()
 	def set_shadow_type(self, type):
 		_gtk.gtk_handle_box_set_shadow_type(self._o, type)
+	def set_handle_position(self, position):
+		_gtk.gtk_handle_box_set_handle_position(self._o, position)
+	def set_snap_edge(self, edge):
+		_gtk.gtk_handle_box_set_snap_edge(self._o, edge)
 
 class GtkItem(GtkBin):
 	get_type = _gtk.gtk_item_get_type
@@ -650,7 +691,7 @@ class GtkCheckMenuItem(GtkMenuItem):
 		}
 		if attrs.has_key(attr):
 			return attrs[attr](self._o)
-		raise AttributeError, attr
+		return GtkMenuItem.__getattr__(self, attr)
 	def set_show_toggle(self, always):
 		_gtk.gtk_check_menu_item_set_show_toggle(self._o, always)
 	def set_active(self, active):
@@ -719,10 +760,9 @@ class GtkViewport(GtkBin):
 	get_type = _gtk.gtk_viewport_get_type
 	def __init__(self, ha=None, va=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if ha or va:
-			self._o = _gtk.gtk_viewport_new(ha._o, va._o)
-		else:
-			self._o = _gtk.gtk_viewport_new()
+		if ha: ha = ha._o
+		if va: va = va._o
+		self._o = _gtk.gtk_viewport_new(ha, va)
 	def get_hadjustment(self):
 		return _obj2inst(_gtk.gtk_viewport_get_hadjustment(self._o))
 	def get_vdjustment(self):
@@ -747,10 +787,6 @@ class GtkWindow(GtkBin):
 		_gtk.gtk_window_activate_default(self._o)
 	def set_title(self, title):
 		_gtk.gtk_window_set_title(self._o, title)
-	def set_focus(self, focus):
-		_gtk.gtk_window_set_focus(self._o, focus._o)
-	def set_default(self, defaultw):
-		_gtk.gtk_window_set_default(self._o, defaultw._o)
 	def set_policy(self, as, ag, autos):
 		_gtk.gtk_window_set_policy(self._o, as, ag, autos)
 	def add_accel_group(self, group):
@@ -772,6 +808,14 @@ class GtkWindow(GtkBin):
 		_gtk.gtk_window_set_icon_name(self._o, name)
 	def set_transient_for(self, parent):
 		_gtk.gtk_window_set_transient_for(self._o, parent._o)
+	def set_geometry_hints(self, geometry_widget=None, **hints):
+		# acceptable hints are min_width, min_height, max_width,
+		# max_height, base_width, base_height, width_inc, height_inc
+		# (with int values) and min_aspect, max_aspect (double values).
+		if hasattr(geometry_widget, '_o'):
+			geometry_widget = geometry_widget._o
+		_gtk.gtk_window_set_geometry_hints(self._o, geometry_widget,
+						   hints)
 	def set_default_size(self, width, height):
 		_gtk.gtk_window_set_default_size(self._o, width, height)
 	def set_modal(self, modal):
@@ -793,7 +837,7 @@ class GtkColorSelectionDialog(GtkWindow):
 		}
 		if attrs.has_key(attr):
 			return _obj2inst(attrs[attr](self._o))
-		raise AttributeError, attr
+		return GtkWindow.__getattr__(self, attr)
 
 class GtkDialog(GtkWindow):
 	get_type = _gtk.gtk_dialog_get_type
@@ -807,7 +851,7 @@ class GtkDialog(GtkWindow):
 		}
 		if attrs.has_key(attr):
 			return _obj2inst(attrs[attr](self._o))
-		raise AttributeError, attr
+		return GtkWindow.__getattr__(self, attr)
 
 class GtkFileSelection(GtkWindow):
 	get_type = _gtk.gtk_file_selection_get_type
@@ -826,8 +870,9 @@ class GtkFileSelection(GtkWindow):
 		'selection_entry': _gtk.gtk_file_selection_get_selection_entry,
 		'selection_text':  _gtk.gtk_file_selection_get_selection_text
 		}
-		if not attrs.has_key(attr): raise AttributeError, attr
-		return _obj2inst(attrs[attr](self._o))
+		if attrs.has_key(attr):
+			return _obj2inst(attrs[attr](self._o))
+		return GtkWindow.__getattr__(self, attr)
 	def set_filename(self, filename):
 		_gtk.gtk_file_selection_set_filename(self._o, filename)
 	def get_filename(self):
@@ -851,7 +896,7 @@ class GtkInputDialog(GtkWindow):
 		}
 		if attrs.has_key(attr):
 			return _obj2inst(attrs[attr](self._o))
-		raise AttributeError, attr
+		return GtkWindow.__getattr__(self, attr)
 
 class GtkFontSelectionDialog(GtkWindow):
 	get_type = _gtk.gtk_font_selection_dialog_get_type
@@ -868,8 +913,9 @@ class GtkFontSelectionDialog(GtkWindow):
 		"cancel_button":
 		        _gtk.gtk_font_selection_dialog_get_cancel_button
 		}
-		if not attrs.has_key(attr): raise AttributeError, attr
-		return _obj2inst(attrs[attr](self._o))
+		if attrs.has_key(attr):
+			return _obj2inst(attrs[attr](self._o))
+		return GtkWindow.__getattr__(self, attr)
 	def get_font_name(self):
 		return _gtk.gtk_font_selection_dialog_get_font_name(self._o)
 	def get_font(self):
@@ -942,7 +988,7 @@ class GtkCombo(GtkHBox):
 	        }
 		if attrs.has_key(attr):
 			return _obj2inst(attrs[attr](self._o))
-		raise AttributeError, attr
+		return GtkHBox.__getattr__(self, attr)
 
 class GtkStatusbar(GtkHBox):
 	get_type = _gtk.gtk_statusbar_get_type
@@ -994,7 +1040,7 @@ class GtkGammaCurve(GtkVBox):
 		}
 		if attrs.has_key(attr):
 			return _obj2inst(attrs[attr](self._o))
-		raise AttributeError, attr
+		return GtkVBox.__getattr__(self, attr)
 
 class GtkButtonBox(GtkBox):
 	get_type = _gtk.gtk_button_box_get_type
@@ -1068,8 +1114,7 @@ class GtkCList(GtkContainer):
 		}
 		if attrs.has_key(attr):
 			return attrs[attr](self._o)
-		else:
-			raise AttributeError, attr
+		return GtkContainer.__getattr__(self, attr)
 	def append(self, values):
 		return _gtk.gtk_clist_append(self._o, values)
 	def clear(self, obj=None):
@@ -1231,8 +1276,7 @@ class GtkCTree(GtkCList):
 		}
 		if attrs.has_key(attr):
 			return attrs[attr](self._o)
-		else:
-			return GtkCList.__getattr__(self, attr)
+		return GtkCList.__getattr__(self, attr)
 	def base_nodes(self):
 		# this returns a list of the base nodes.  Useful for recursion
 		return _gtk.gtk_ctree_base_nodes(self._o)
@@ -1497,7 +1541,9 @@ class GtkMenu(GtkMenuShell):
 	def insert(self, child, pos):
 		_gtk.gtk_menu_insert(self._o, child._o, pos)
 	def popup(self, pms, pmi, func, button, time):
-		_gtk.gtk_menu_popup(self._o, pms._o, pmi._o, func,button,time)
+		if pms: pms = pms._o
+		if pmi: pmi = pmi._o
+		_gtk.gtk_menu_popup(self._o, pms, pmi, func, button, time)
 	def reposition(self):
 		_gtk.gtk_menu_reposition(self._o)
 	def popdown(self, obj=None):
@@ -1673,6 +1719,8 @@ class GtkPaned(GtkContainer):
 		_gtk.gtk_paned_pack1(self._o, child._o, resize, shrink)
 	def pack2(self, child, resize=TRUE, shrink=TRUE):
 		_gtk.gtk_paned_pack2(self._o, child._o, resize, shrink)
+	def set_position(self, position):
+		_gtk.gtk_paned_set_position(self._o, position)
 	def handle_size(self, size):
 		print "handle_size deprecated -- use set_handle_size"
 		self.set_handle_size(size)
@@ -1700,10 +1748,9 @@ class GtkScrolledWindow(GtkBin):
 	get_type = _gtk.gtk_scrolled_window_get_type
 	def __init__(self, hadj=None, vadj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if not (vadj or hadj):
-			self._o = _gtk.gtk_scrolled_window_new()
-		else:
-			self._o = _gtk.gtk_scrolled_window_new(hadj._o, vadj._o)
+		if hadj: hadj = hadj._o
+		if vadj: vadj = vadj._o
+		self._o = _gtk.gtk_scrolled_window_new(hadj, vadj)
 	def get_hadjustment(self):
 		return _obj2inst(_gtk.gtk_scrolled_window_get_hadjustment(
 			self._o))
@@ -1959,11 +2006,8 @@ class GtkSpinButton(GtkEntry):
 	def __init__(self, adj=None, climb_rate=1, digits=1, _obj=None):
 		if _obj: self._o = _obj; return
 		if adj:
-			self._o = _gtk.gtk_spin_button_new(adj._o, climb_rate,
-							   digits)
-		else:
-			self._o = _gtk.gtk_spin_button_new_no_adj(climb_rate,
-								  digits)
+			adj = adj._o
+		self._o = _gtk.gtk_spin_button_new(adj, climb_rate, digits)
 	def set_adjustment(self, adj):
 		_gtk.gtk_spin_button_set_adjustment(self._o, adj._o)
 	def get_adjustment(self):
@@ -1996,10 +2040,9 @@ class GtkText(GtkEditable):
 	get_type = _gtk.gtk_text_get_type
 	def __init__(self, hadj=None, vadj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if not (hadj or vadj):
-			self._o = _gtk.gtk_text_new()
-		else:
-			self._o = _gtk.gtk_text_new(hadj._o, vadj._o)
+		if hadj: hadj = hadj._o
+		if vadj: vadj = vadj._o
+		self._o = _gtk.gtk_text_new(hadj, vadj)
 	def set_editable(self, editable):
 		_gtk.gtk_text_set_editable(self._o, editable)
 	def set_word_wrap(self, word_wrap):
@@ -2249,19 +2292,15 @@ class GtkHScale(GtkScale):
 	get_type = _gtk.gtk_hscale_get_type
 	def __init__(self, adj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if adj:
-			self._o = _gtk.gtk_hscale_new(adj._o)
-		else:
-			self._o = _gtk.gtk_hscale_new()
+		if adj: adj = adj._o
+		self._o = _gtk.gtk_hscale_new(adj)
 
 class GtkVScale(GtkScale):
 	get_type = _gtk.gtk_vscale_get_type
 	def __init__(self, adj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if adj:
-			self._o = _gtk.gtk_vscale_new(adj._o)
-		else:
-			self._o = _gtk.gtk_vscale_new()
+		if adj: adj = adj._o
+		self._o = _gtk.gtk_vscale_new(adj)
 
 class GtkScrollbar(GtkRange):
 	get_type = _gtk.gtk_scrollbar_get_type
@@ -2272,19 +2311,15 @@ class GtkHScrollbar(GtkScrollbar):
 	get_type = _gtk.gtk_hscrollbar_get_type
 	def __init__(self, adj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if adj:
-			self._o = _gtk.gtk_hscrollbar_new(adj._o)
-		else:
-			self._o = _gtk.gtk_hscrollbar_new()
+		if adj: adj = adj._o
+		self._o = _gtk.gtk_hscrollbar_new(adj)
 
 class GtkVScrollbar(GtkScrollbar):
 	get_type = _gtk.gtk_vscrollbar_get_type
 	def __init__(self, adj=None, _obj=None):
 		if _obj: self._o = _obj; return
-		if adj:
-			self._o = _gtk.gtk_vscrollbar_new(adj._o)
-		else:
-			self._o = _gtk.gtk_vscrollbar_new()
+		if adj: adj = adj._o
+		self._o = _gtk.gtk_vscrollbar_new(adj)
 
 class GtkRuler(GtkWidget):
 	get_type = _gtk.gtk_ruler_get_type
